unit LPingerU;

interface

uses
  Common, DateUtils, MkPinger, ShellAPI,
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ExtCtrls, Buttons, IdBaseComponent, IdComponent,
  IdRawBase, IdRawClient, IdIcmpClient, Spin, ComCtrls, ToolWin, ImgList,
  ActnList;

type
  TLPingerF = class(TForm)
    Panel2: TPanel;
    Panel3: TPanel;
    HostsLB: TListBox;
    Timer: TTimer;
    PingIntervalSE: TSpinEdit;
    ReceiveTimeoutSE: TSpinEdit;
    PingsToHostSE: TSpinEdit;
    PingIntervalL: TLabel;
    ReceiveTimeoutL: TLabel;
    PingsToHostL: TLabel;
    DateTimeP: TPanel;
    ToolB: TToolBar;
    StartStopTB: TToolButton;
    ImageL: TImageList;
    ActionL: TActionList;
    ToolButton2: TToolButton;
    StartStopA: TAction;
    StartStopP: TPanel;
    StartStopL: TLabel;
    StartStopDTL: TLabel;
    ToolButton1: TToolButton;
    LocalLogsA: TAction;
    ToolButton3: TToolButton;
    ToolButton4: TToolButton;
    GraphA: TAction;
    ToolButton5: TToolButton;
    ToolButton6: TToolButton;
    ToolButton7: TToolButton;
    TrayA: TAction;
    CloseA: TAction;
    PingingP: TPanel;
    ToolButton8: TToolButton;
    ToolButton9: TToolButton;
    HostEditA: TAction;
    ToolButton10: TToolButton;
    ClearLogsA: TAction;
    ToolButton11: TToolButton;
    AboutA: TAction;
    procedure FormDestroy(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure RefreshHostsSBClick(Sender: TObject);
    procedure TimerTimer(Sender: TObject);
    procedure ReceiveTimeoutSEChange(Sender: TObject);
    procedure ICMPReply(ASender: TComponent; const ReplyStatus: TReplyStatus);
    procedure ProcessLogsSBClick(Sender: TObject);
    procedure PingsToHostSEChange(Sender: TObject);
    procedure PingsFinished(Sender: TObject);
    procedure AbortBBClick(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure GraphBBClick(Sender: TObject);
    procedure OkBBClick(Sender: TObject);
    procedure StartStopAExecute(Sender: TObject);
    procedure LocalLogsAExecute(Sender: TObject);
    procedure GraphAExecute(Sender: TObject);
    procedure TrayAExecute(Sender: TObject);
    procedure CloseAExecute(Sender: TObject);
    procedure HostEditAExecute(Sender: TObject);
    procedure ClearLogsAExecute(Sender: TObject);
    procedure AboutAExecute(Sender: TObject);
  private
    FPingerList : TMkPingerList;
    FPngSettings : TLPingerSettings;
    FPinging: boolean;
    FPinged : boolean;
    IconData: TNotifyIconData;
    FPingEnabled: boolean; { System tray icon }
    procedure RefreshHosts;
    procedure ProcessHost( H : string);
    procedure ProcessAllHosts;
    procedure SetPinging(const Value: boolean);
    procedure SetPngSettings(const Value: TLPingerSettings);
    procedure CorrectSettings;
    procedure SendToTray;
    procedure RestoreFromTray;
    procedure TrayEventHandler(var Msg: TMessage); message WM_USER + 1;
    procedure SetPingEnabled(const Value: boolean);
  public
    property Pinging : boolean read FPinging write SetPinging;
    property PngSettings : TLPingerSettings read FPngSettings write SetPngSettings;
    property PingEnabled : boolean read FPingEnabled write SetPingEnabled;
  end;

var
  LPingerF: TLPingerF;

implementation
{$R *.dfm}

uses LPLogViewU, LPingGraphU, HostEditorU, About;

procedure TLPingerF.CorrectSettings;
begin

end;

procedure TLPingerF.FormCreate(Sender: TObject);
begin
  Pinging := false;
  FPinged := false;
  StartStopDTL.Caption := FormatDateTime( 'dd-mm-yy  hh:nn:ss', Now);
  Timer.Enabled := false;

  Application.Title := 'Initializing...';

  if NOT DirectoryExists( LogsSubdir) then  CreateDir( LogsSubdir);
  FPingerList := TMkPingerList.Create;
  FPingerList.OnFinish := PingsFinished;

  RefreshHosts;
  ReadSettings( PngSettings);

  PingsToHostSEChange( Self);
  ReceiveTimeoutSEChange( Self);

  FPingEnabled := false; PingEnabled := true;

  Timer.Enabled := true;
end;

procedure TLPingerF.FormDestroy(Sender: TObject);
begin
  Timer.Enabled := false;
  FPingerList.Free;
end;

procedure TLPingerF.ProcessAllHosts;
var Cou : integer;
begin
  Timer.Enabled := false;
  try FPingerList.Process;
  except end;
  PingingP.Caption := 'Waiting ...';
  Application.ProcessMessages;
end;

procedure TLPingerF.ProcessHost(H: string);
var Cou : integer;
begin
end;

procedure TLPingerF.RefreshHosts;
var SL : TStringList; Cou : integer; MKP : TMkPinger;
begin
  Timer.Enabled := false;
  SL := TStringList.Create;
  FPingerList.Clear;
  try
    ReadHosts( SL);
    ActualHosts( SL, HostsLB.Items);
    for Cou := 0 to HostsLB.Items.Count - 1 do begin
      MKP := FPingerList.AddNewPing;
      MKP.Host := HostsLB.Items.Strings[ Cou];
      MKP.OnEachReply := ICMPReply;
      end;
  finally
    SL.Free;
    end;
end;

procedure TLPingerF.RefreshHostsSBClick(Sender: TObject);
begin
  RefreshHosts;
  Timer.Enabled := true;
end;

procedure TLPingerF.SetPinging(const Value: boolean);
begin
  FPinging := Value;

  CloseA.Enabled := NOT FPinging;
  TrayA.Enabled := NOT FPinging;
  LocalLogsA.Enabled := NOT FPinging;
  GraphA.Enabled := NOT FPinging;
  StartStopA.Enabled := NOT FPinging;
  HostEditA.Enabled := NOT FPinging;
  ClearLogsA.Enabled := NOT FPinging;

  PingIntervalSE.Enabled := NOT FPinging;   PingIntervalL.Enabled := NOT FPinging;
  ReceiveTimeoutSE.Enabled := NOT FPinging; ReceiveTimeoutL.Enabled := NOT FPinging;
  PingsToHostSE.Enabled := NOT FPinging;    PingsToHostL.Enabled := NOT FPinging;

  if FPinging then begin
    Application.Icon.Handle := LoadIcon(hInstance, 'ACTIVE');
    Application.Title := 'Pinging...';
    end
  else begin
    Application.Icon.Handle := LoadIcon(hInstance, 'MAINICON');
    PingingP.Caption := 'Waiting ...';
    end;
  Application.ProcessMessages;
end;

procedure TLPingerF.SetPngSettings(const Value: TLPingerSettings);
begin
//  if FPngSettings = Value then exit;
  FPngSettings := Value;
  PingIntervalSE.Value   := FPngSettings.ProcessInterval;
  ReceiveTimeoutSE.Value := FPngSettings.ReceiveTimeout;
  PingsToHostSE.Value    := FPngSettings.PingsToHost;
  CorrectSettings;
end;

procedure TLPingerF.TimerTimer(Sender: TObject);
var NowDT : TDateTime; WaitMin : integer;
begin
  NowDT := Now;
  DateTimeP.Caption := FormatDateTime( 'dd-mm-yy  hh:nn:ss', NowDT);

  if ( PingEnabled) AND
{     ( MinuteOfTheYear( NowDT) MOD PingIntervalSE.Value = 0) AND}
     ( SecondOfTheYear( NowDT) MOD ( PingIntervalSE.Value * 60) = 0) AND
     ( NOT Pinging){ AND ( NOT FPinged) }then begin
    Pinging := true;
    ProcessAllHosts;  // ping all hosts
    FPinged := true;
//    Pinging := false;
    end;

  // reset pinged flag
  if ( MinuteOfTheYear( NowDT) MOD PingIntervalSE.Value <> 0) then FPinged := false;

  if NOT FPinging then begin
    WaitMin := PingIntervalSE.Value - MinuteOfTheYear( NowDT) MOD PingIntervalSE.Value;
    PingingP.Caption := 'Next process in ' +
                        IntToStr( WaitMin) +
                        ' minutes';
    Application.Title := 'Waiting ' + IntToStr(  WaitMin) + ' min';
    end;
end;

procedure TLPingerF.ReceiveTimeoutSEChange(Sender: TObject);
begin
  FPingerList.Timeout := ReceiveTimeoutSE.Value * 1000;
end;

procedure TLPingerF.ICMPReply(ASender: TComponent;
  const ReplyStatus: TReplyStatus);
begin
  PingingP.Caption := 'Pinging...';
  if Assigned( LPLogViewF) then
    {if LPLogViewF.Visible then }LPLogViewF.ShowLogs( ReplyStatus);
  Application.ProcessMessages;    
end;

procedure TLPingerF.ProcessLogsSBClick(Sender: TObject);
begin
  if Assigned( LPLogViewF) then begin
    LPLogViewF.Visible := true;
    end;
end;

procedure TLPingerF.PingsToHostSEChange(Sender: TObject);
begin
  FPingerList.PingsToHost := PingsToHostSE.Value;
end;

procedure TLPingerF.PingsFinished(Sender: TObject);
var Cou : integer;
begin
  Timer.Enabled := true;
  for Cou := 0 to FPingerList.Count - 1 do begin
    FPingerList.PingItems[ Cou].WriteLog;
    end;
  Pinging := false;
end;

procedure TLPingerF.AbortBBClick(Sender: TObject);
begin
  Close;
end;

procedure TLPingerF.FormCloseQuery(Sender: TObject; var CanClose: Boolean);
begin
  if MessageDlg('Do you really want ABORT pinger?', mtWarning, [mbYes, mbNo, mbCancel], 0) = mrYes then begin
    Timer.Enabled := false;
    CanClose := true;
    end
  else CanClose := false;
end;

procedure TLPingerF.GraphBBClick(Sender: TObject);
begin
  LPingGraphF := TLPingGraphF.Create( Application);
  LPingGraphF.ShowModal;
  LPingGraphF.Free;
end;

procedure TLPingerF.OkBBClick(Sender: TObject);
begin
  SendToTray;
//  Application.Minimize;
end;

procedure TLPingerF.RestoreFromTray;
begin
  Shell_NotifyIcon(NIM_DELETE, @IconData); { Remove icon from system tray }
  { Show form and taskbar button }
  self.Show;
  ShowWindow(Application.Handle, SW_NORMAL)
end;

procedure TLPingerF.SendToTray;
begin
  with IconData do begin { Define structure of tray icon }
    cbSize:= SizeOf(IconData); { Size of this structure }
    Wnd:= LPingerF.Handle; { Window handle to send messages to }
    uID:= 1; { Some icon identifier }
    uFlags:=  NIF_ICON or NIF_MESSAGE or NIF_TIP; { Things we are specifing in this structure }
    uCallbackMessage := WM_USER + 1; { Application message that will be send back }
    hIcon:= Application.Icon.Handle; { Handle of icon to show }
    szTip:= 'Local pinger' { Tooltip of tray icon }
    end;
  Shell_NotifyIcon(NIM_ADD, @IconData); { Add icon to system tray }
  { Hide form and taskbar button }
  self.Hide;
  ShowWindow(Application.Handle, SW_HIDE);
end;

procedure TLPingerF.TrayEventHandler(var Msg: TMessage);
begin
  if Msg.lParam = WM_LBUTTONDBLCLK then RestoreFromTray;
end;

procedure TLPingerF.SetPingEnabled(const Value: boolean);
begin
  if FPingEnabled = Value then exit;
  FPingEnabled := Value;
  if FPingEnabled then begin
    StartStopA.ImageIndex := 1;
    StartStopA.Hint := 'Stop';
    StartStopL.Caption := 'Started at:';
    end
  else begin
    StartStopA.ImageIndex := 0;
    StartStopA.Hint := 'Start';
    StartStopL.Caption := 'Stopped';
    end;
  StartStopDTL.Visible := FPingEnabled;
end;

procedure TLPingerF.StartStopAExecute(Sender: TObject);
begin
  PingEnabled := NOT PingEnabled;
end;

procedure TLPingerF.LocalLogsAExecute(Sender: TObject);
begin
  if Assigned( LPLogViewF) then begin
    LPLogViewF.Visible := true;
    end;
end;

procedure TLPingerF.GraphAExecute(Sender: TObject);
begin
  LPingGraphF := TLPingGraphF.Create( Application);
  LPingGraphF.ShowModal;
  LPingGraphF.Free;
end;

procedure TLPingerF.TrayAExecute(Sender: TObject);
begin
  SendToTray;
end;

procedure TLPingerF.CloseAExecute(Sender: TObject);
begin
  Close;
end;

procedure TLPingerF.HostEditAExecute(Sender: TObject);
var
  MResult : integer;
begin
  HostEditorF := THostEditorF.Create( Application);
  MResult := HostEditorF.ShowModal;
  HostEditorF.Free;
  if MResult = mrOk then begin
    RefreshHosts;
    Timer.Enabled := true;
    end;
end;

procedure TLPingerF.ClearLogsAExecute(Sender: TObject);
var SR: TSearchRec;
begin
  if MessageDlg('Clear logs and ping data?',
      mtConfirmation, [mbYes, mbNo], 0) <> mrYes then exit;
  if FindFirst(  LogsSubdir + '*.*', faAnyFile - faDirectory - faVolumeID, SR) = 0 then begin
//    ShowInfo( 'Deleting: ' + LogsSubdir + SR.Name);
    Application.ProcessMessages;
    DeleteFile( LogsSubdir + SR.Name);
    while FindNext( SR) = 0 do begin
//      ShowInfo( 'Deleting: ' + Dir + SR.Name);
      Application.ProcessMessages;
      DeleteFile( LogsSubdir + SR.Name);
      end;
    end;
  FindClose( SR);
end;

procedure TLPingerF.AboutAExecute(Sender: TObject);
begin
  AboutBox := TAboutBox.Create( Application);
  AboutBox.ShowModal;
  AboutBox.Free;
end;

end.
